struct VS_RESULT {
  float4 pos:       POSITION;
  float2 texCoord:  TEXCOORD0;
  float3 position:  TEXCOORD1;
  float3 lightDir:  TEXCOORD2;
  float3 halfAngle: TEXCOORD3;
};

uniform float4   lightPosition;
uniform float4x4 worldViewProj;
uniform float4x4 worldView;

VS_RESULT main(	float4 pos: POSITION,
		float3 normal: NORMAL,
		float2 texCoord0: TEXCOORD0,
		float3 tangent: TANGENT,
                float3 binormal : BINORMAL
		) {
   VS_RESULT result;

   float4 position   = mul(worldView, pos);
   
   result.pos        = mul(worldViewProj, pos);
   result.texCoord   = texCoord0;
   result.position   = position.xyz / position.w;

   float3 nnormal    = normalize(mul((float3x3) worldView, normal));
   float3 ntangent   = normalize(mul((float3x3) worldView, tangent));
   float3 nbinormal  = normalize(cross(nnormal, ntangent));
   
   float3x3 normalMatrix = float3x3(ntangent, nbinormal, nnormal);
   
   result.lightDir   = normalize((float3) lightPosition - result.position);
   result.halfAngle  = -1.0f * result.position + result.lightDir;
   
   result.lightDir   = normalize(mul(normalMatrix, result.lightDir));
   result.halfAngle  = normalize(mul(normalMatrix, result.halfAngle));
   
   return result;
}